import os.path
import subprocess

from django.conf import settings

from .base import Backend, CompileError


class MjmlBackend(Backend):
    def compile(self, source):
        command = ['-i', '-s']
        if isinstance(settings.HAPPYMAILER_MJML_BIN, list):
            command = [os.path.expanduser(x) for x in settings.HAPPYMAILER_MJML_BIN] + command
        else:
            command.insert(0, os.path.expanduser(settings.HAPPYMAILER_MJML_BIN))
        proc = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, _ = proc.communicate(source.encode('utf-8'))
        out = out.decode('utf-8')
        if proc.returncode:
            raise CompileError(out)
        if 'node_modules/mjml' in out:
            raise CompileError(out)
        return out



from django.test import SimpleTestCase

from ..backends import MjmlBackend, CompileError


class MjmlCase(SimpleTestCase):
    def test_mjml(self):
        backend = MjmlBackend()
        backend.compile('<mjml><mj-body><mj-button>foo</mj-button></mj-body></mjml>')

        with self.assertRaises(CompileError):
            backend.compile('foo')



from django.apps import AppConfig

__all__ = ('HappyMailerConfig',)


class HappymailerConfig(AppConfig):
    name = 'happymailer'

    def ready(self):
        # noinspection PyUnresolvedReferences
        from . import receivers
        from .utils import find_templates, all_template_classes, all_layout_classes
        find_templates()
        for cls in all_template_classes:
            cls.check()
        for cls in all_layout_classes:
            cls.check()



class Backend(object):
    def compile(self, source):
        raise NotImplementedError()


class CompileError(Exception):
    pass


class InvalidVariableException(object):
    def __mod__(self, missing):
        raise CompileError('Unknown template variables: %s' % missing)

    def __contains__(self, item):
        if item == '%s':
            return True
        return False


from django.core.management.base import BaseCommand
from ...utils import template_compile_check
from ...backends.base import CompileError

class Command(BaseCommand):
    """
    create db templates
    """
    can_import_settings = True
    requires_system_checks = True
    output_transaction = True

    def add_arguments(self, parser):
        parser.add_argument('--no-check', action='store_true', dest='no_check', default=False,
                            help='disable templates compile check')

    def handle(self, *args, **options):
        from ...utils import template_classes
        from ...models import TemplateModel
        for cls in template_classes:
            if not TemplateModel.objects.filter(name=cls.name).exists():
                try:
                    cls.check()
                except AssertionError:
                    pass
                else:
                    self.stdout.write('create {} ({}.{})\n'.format(cls.name, cls.__module__, cls.__name__))
                    TemplateModel.objects.create(name=cls.name)

            if not options.get('no_check'):
                try:
                    template_compile_check(cls)
                except CompileError as e:
                    self.stdout.write('{} got compile error: {}'.format(cls.name, str(e)))


import six
from django.core import mail
from django.core.management import call_command
from django.db import transaction
from django.test import TestCase, TransactionTestCase, override_settings

from dummy.mails import WelcomeTemplate, BasicLayout
from dummy2.mails import DummyTemplate
from .. import Template, t, Layout
from ..models import TemplateModel
from ..utils import (
    template_classes, layout_classes, find_templates,
    TemplateConfigurationError, LayoutNotFound,
)


class BasicsCase(TestCase):
    def test_abstract(self):
        class TestTemplate(Template):
            name = 'foo'
            abstract = True

        with self.assertRaises(AssertionError):
            TestTemplate.check()

        with self.assertRaises(AssertionError):
            TestTemplate('spam')

    def test_defined(self):
        with self.assertRaises(AssertionError):
            class TestTemplate(Template):
                name = 'foo'

            TestTemplate.check()

        with self.assertRaises(AssertionError):
            class TestTemplate2(Template):
                variables = {}

            TestTemplate2.check()

    def test_empty(self):
        class TestTemplate(Template):
            name = 'foo'
            variables = {}

        TemplateModel.objects.create(name='foo', layout='basic')

        tmpl = TestTemplate('spam')
        self.assertDictEqual(tmpl.kwargs, {})

    def test_trafaret(self):
        class TestTemplate(Template):
            name = 'test'
            variables = {'foo': t.Int()}
            kwargs = {'bar': t.String()}

            def get_variables(self):
                return {
                    'foo': int(self.kwargs['bar'])
                }

        TemplateModel.objects.create(name='test', layout='basic')

        tmpl = TestTemplate('spam', bar='1')
        self.assertDictEqual(tmpl.variables, {'foo': 1})

    def test_kwargs(self):
        class TestTemplate(Template):
            name = 'test'
            variables = {}
            kwargs = {'foo': t.String(), 'spam': t.Int()}

        TemplateModel.objects.create(name='test', layout='basic')

        tmpl = TestTemplate('spam', foo='bar', spam=1)
        self.assertDictEqual(tmpl.kwargs, {'foo': 'bar', 'spam': 1})

        with self.assertRaises(t.DataError):
            TestTemplate('spam', foo='bar')

        with self.assertRaises(t.DataError):
            TestTemplate('spam', foo='bar', spam='eggs')

    def test_find_templates(self):

        with override_settings(INSTALLED_APPS=[]):
            self.assertListEqual(template_classes, [])
            self.assertListEqual(layout_classes, [])

        with override_settings(INSTALLED_APPS=['dummy']):
            self.assertListEqual(template_classes, [WelcomeTemplate])
            self.assertListEqual(layout_classes, [BasicLayout])

        with override_settings(INSTALLED_APPS=['dummy2']):
            self.assertListEqual(template_classes, [DummyTemplate])
            self.assertListEqual(layout_classes, [])

        with override_settings(INSTALLED_APPS=['dummy', 'dummy2']):
            self.assertSetEqual(set(template_classes), {WelcomeTemplate, DummyTemplate})
            self.assertSetEqual(set(layout_classes), {BasicLayout})


class EnabledCase(TestCase):
    def test_transaction(self):
        class TestTemplate(Template):
            name = 'test'
            variables = {}

        with self.assertRaises(TemplateConfigurationError):
            TestTemplate('spam')

        model = TemplateModel.objects.create(name='test')

        with self.assertRaises(TemplateConfigurationError):
            TestTemplate('spam')

        model.layout = 'invalid layout'
        model.save()

        with self.assertRaises(LayoutNotFound):
            TestTemplate('spam')

        model.layout = 'basic'
        model.save()

        self.assertFalse(TestTemplate('spam').enabled)

        model.enabled = True
        model.save()

        self.assertTrue(TestTemplate('spam').enabled)


class MigrationCase(TransactionTestCase):
    def test_created(self):
        stdout = six.StringIO()
        call_command('create_email_templates', no_check=True, stdout=stdout)
        self.assertIn('welcome', stdout.getvalue())
        self.assertIn('dummy', stdout.getvalue())
        self.assertTrue(TemplateModel.objects.filter(name='welcome').exists())
        self.assertTrue(TemplateModel.objects.filter(name='dummy').exists())


class HistoryCase(TransactionTestCase):
    def test_version(self):
        model = TemplateModel.objects.create(name='foo')
        self.assertEqual(model.version, 1)

        model.save()
        self.assertEqual(model.version, 1)
        model.refresh_from_db()
        self.assertEqual(model.version, 1)

        model.body = 'spam'
        model.save()
        self.assertEqual(model.version, 2)
        model.refresh_from_db()
        self.assertEqual(model.version, 2)

    def test_history(self):
        with transaction.atomic():
            model = TemplateModel.objects.create(name='foo')

        self.assertEqual(model.history.count(), 0)

        with transaction.atomic():
            model.save()

        self.assertEqual(model.history.count(), 0)

        with transaction.atomic():
            model.body = 'spam'
            model.save()

        self.assertEqual(model.history.count(), 1)


class LayoutCase(TestCase):
    def test_abstract(self):
        class AbstractLayout(Layout):
            abstract = True

        with self.assertRaises(AssertionError):
            AbstractLayout()

    def test_check(self):
        class InvalidLayout(Layout):
            pass

        class AbstractLayout(Layout):
            abstract = True

        class SomeLayout(Layout):
            name = 'some'

        with self.assertRaises(AssertionError):
            InvalidLayout.check()

        self.assertIsNone(AbstractLayout.check())
        self.assertIsNone(SomeLayout.check())


    def test_variables(self):
        class TestLayout(Layout):
            variables = {'foo': t.String()}

            def get_variables(self):
                return {'foo': 'bar'}

        self.assertDictEqual(TestLayout().variables, {'foo': 'bar'})


class ComplexCase(TestCase):
    def test_complex(self):
        class ComplexLayout(Layout):
            name = 'complex'
            variables = {'foo': t.String()}

            def render(self, content=None):
                return dict(self.variables)

            def get_variables(self):
                return {
                    'foo': 'bar'
                }

        class ComplexTemplate(Template):
            name = 'complex'
            kwargs = {'foo': t.String(), 'spam': t.String()}
            variables = {'spam': t.String()}

            def render(self):
                layout = self.layout_cls()
                return layout.render(), dict(self.variables)

            def get_variables(self):
                return {
                    'spam': self.kwargs['spam']
                }

        TemplateModel.objects.create(name='complex', layout='complex')

        find_templates()

        tmpl = ComplexTemplate('spam', foo='bar', spam='eggs')

        layout_render, template_render = tmpl.render()
        self.assertDictEqual(layout_render, {'foo': 'bar'})
        self.assertDictEqual(template_render, {'spam': 'eggs'})


class RenderCase(TestCase):
    def test_render(self):
        class EmptyLayout(Layout):
            name = 'empty'
            content = '{{ body }}'

        class WrapLayout(Layout):
            name = 'wrap'
            content = 'wrap{{ body }}wrap'

        class TestTemplate(Template):
            name = 'test'
            variables = {'count': t.Any()}
            kwargs = {'count': t.Int()}

            def get_variables(self):
                return {
                    'count': self.kwargs['count']
                }

        find_templates()

        model = TemplateModel.objects.create(name='test', body='', layout='empty')

        tmpl = TestTemplate('spam', count=3)
        self.assertEqual(tmpl.render(), '')

        model.body = '{{ count }}test'
        model.save()

        tmpl = TestTemplate('spam', count=3)
        self.assertEqual(tmpl.render(), '3test')

        model.layout = 'wrap'
        model.save()

        tmpl = TestTemplate('spam', count=3)
        self.assertEqual(tmpl.render(), 'wrap3testwrap')


@override_settings(
    EMAIL_BACKEND='django.core.mail.backends.locmem.EmailBackend'
)
class CompileCase(TestCase):
    def setUp(self):
        class TestLayout(Layout):
            name = 'test'
            content = '<mjml><mj-body>{{ body }}</mj-body></mjml>'

        class TestTemplate(Template):
            name = 'test'
            variables = {'btn': t.String(), 'name': t.String()}
            kwargs = {'btn': t.String(), 'name': t.String()}

            def get_variables(self):
                return self.kwargs

        TemplateModel.objects.create(
            name='test', layout='test',
            subject='hello, {{ name }}!',
            body='''
            <mj-section>
              <mj-button href="https://github.com/">{{ btn }}</mj-button>
            </mj-section>
            '''
        )

        find_templates()

        self.tmpl = TestTemplate('spam', name='world', btn='foo bar spam eggs')

    def test_compile(self):
        self.assertIn('foo bar spam eggs', self.tmpl.compile())

    def test_send_and_plaintext(self):
        self.tmpl.send(force=True)
        self.assertEqual(mail.outbox[0].subject, 'hello, world!')
        self.assertIn('foo bar spam eggs', mail.outbox[0].body)
        self.assertIn('https://github.com/', mail.outbox[0].body)



import logging

import html2text
import six
import trafaret as t
from django.conf import settings
from django.core.mail import send_mail
from django.template import Template as DjangoTemplate, Context
from django.utils.functional import cached_property
from django.utils.module_loading import import_string

from . import fake
from .backends.base import InvalidVariableException
from .utils import all_template_classes, all_layout_classes, get_layout, TemplateConfigurationError

__all__ = ('Template', 'Layout', 't')

logger = logging.getLogger(__name__)


class BasicMeta(type):
    registry = []
    ignore_extra = False

    def __new__(mcs, name, bases, dct):
        dct.setdefault('abstract', False)
        abstract = dct['abstract']

        base_variables = t.Dict({})

        for klass in bases:
            variables = getattr(klass, 'variables', None)
            if variables:
                base_variables = base_variables.merge(variables)

        if 'kwargs' in dct:
            kwargs = dct.pop('kwargs')
            if not abstract or kwargs is not None:
                assert isinstance(kwargs, dict)
                kwargs = t.Dict(**kwargs)
                if mcs.ignore_extra:
                    kwargs = kwargs.ignore_extra('*')

            dct.update(kwargs=kwargs)

        if 'variables' in dct:
            variables = dct.pop('variables')
            if not abstract or variables is not None:
                assert isinstance(variables, dict)
                variables = base_variables.merge(variables)
                if mcs.ignore_extra:
                    variables = variables.ignore_extra('*')

            dct.update(variables=variables)

        cls = type.__new__(mcs, name, bases, dct)

        if not abstract:
            mcs.registry.append(cls)

        return cls


class LayoutMeta(BasicMeta):
    registry = all_layout_classes
    ignore_extra = True


class TemplateMeta(BasicMeta):
    registry = all_template_classes


class MessageMeta(BasicMeta):
    pass


class Layout(six.with_metaclass(LayoutMeta)):
    name = None
    description = None
    abstract = True
    variables = {}
    content = None

    def __init__(self):
        assert not self.abstract
        self.variables = self.variables.check_and_return(self.get_variables())

    def get_variables(self):
        return {}

    def render(self, content):
        pass

    @classmethod
    def check(cls):
        if cls.abstract:
            assert not cls.name, 'abstract layout {}.{} must not have a name'.format(cls.__module__, cls.__name__)
        else:
            assert cls.name


class Template(six.with_metaclass(TemplateMeta)):
    name = None
    variables = None
    kwargs = {}
    abstract = True

    def __init__(self, recipient=None, _force_layout_cls=None, _force_variables=None, **kwargs):
        assert not self.abstract

        self._recipients = []
        if recipient:
            self._recipients.append(recipient)

        if _force_layout_cls:
            self.layout_cls = _force_layout_cls
        else:
            self.layout_cls = None
            if self.layout:
                self.layout_cls = get_layout(self.layout)

        if not self.layout_cls:
            raise TemplateConfigurationError('no layout specified for {} template'.format(self.name))

        self.kwargs = self.kwargs.check_and_return(kwargs)

        if _force_variables:
            self.variables = _force_variables
        else:
            self.post_init()
            self.variables = self.variables.check_and_return(self.get_variables())

    def post_init(self):
        pass

    def add_recipient(self, recipient):
        self._recipients.append(recipient)

    def recipients(self):
        return self._recipients

    @classmethod
    def fake_variables(cls):
        return fake.generate(cls.variables)

    @classmethod
    def fake_kwargs(cls):
        return fake.generate(cls.kwargs)

    @cached_property
    def model(self):
        from .models import TemplateModel
        return TemplateModel.objects.filter(name=self.name).first()

    @cached_property
    def layout(self):
        return self.model.layout if self.model else None

    @cached_property
    def subject(self):
        return self.model.subject if self.model else None

    @cached_property
    def enabled(self):
        return self.model.enabled if self.model else None

    @cached_property
    def body(self):
        return self.model.body if self.model else None

    def get_variables(self):
        return {}

    def render(self):
        layout = self.layout_cls()
        dj_tmpl = DjangoTemplate(self.body)
        dj_tmpl.engine.string_if_invalid = InvalidVariableException()
        body = dj_tmpl.render(Context(self.variables))
        dj_tmpl.engine.string_if_invalid = ''
        return six.text_type(DjangoTemplate(layout.content).render(Context(dict(layout.variables, body=body))))

    def compile(self):
        backend_cls = import_string(settings.HAPPYMAILER_BACKEND)
        backend = backend_cls()
        return backend.compile(self.render())

    def send(self, force=False):
        if not self.enabled and not force:
            return
        subject = six.text_type(DjangoTemplate(self.subject).render(Context(self.variables)))
        html = self.compile()
        text = html2text.html2text(html)
        self._send(subject, text, settings.HAPPYMAILER_FROM, recipient_list=self.recipients(),
                   html_message=html, fail_silently=False)

    def _send(self, subject, message, from_email, recipient_list,
              fail_silently=False, auth_user=None, auth_password=None,
              connection=None, html_message=None):
        send_mail(subject, message, from_email, recipient_list,
                  fail_silently, auth_user, auth_password,
                  connection, html_message)

    @classmethod
    def check(cls):
        if cls.abstract:
            assert not cls.name, 'abstract template {}.{} must not have a name'.format(cls.__module__, cls.__name__)
        else:
            assert cls.name
            assert cls.variables
            assert cls.kwargs



import logging

from django.core.signals import setting_changed
from django.db import models
from django.db import transaction
from django.dispatch import receiver

from .models import TemplateModel
from .utils import find_templates

logger = logging.getLogger(__name__)


@receiver(models.signals.post_init, sender=TemplateModel)
@receiver(models.signals.post_save, sender=TemplateModel)
def update_db_state(instance, **kwargs):
    instance.db_subject = instance.subject
    instance.db_body = instance.body
    instance.db_layout = instance.layout
    instance.db_version = instance.version


@receiver(models.signals.pre_save, sender=TemplateModel)
def update_history(instance, **kwargs):
    if instance.body != instance.db_body \
            or instance.layout != instance.db_layout \
            or instance.subject != instance.db_subject:
        instance.version += 1

    if instance.version != instance.db_version:
        version = instance.version

        @transaction.on_commit
        def save():
            instance.history.create(
                version=version,
                body=instance.body,
                subject=instance.subject,
                layout=instance.layout,
            )


@receiver(setting_changed)
def reload_templates(**kwargs):
    find_templates()



import random

import trafaret as t
from django.conf import settings
from faker import Factory

fake = Factory.create(settings.LANGUAGE_CODE)


def generate(trafaret):
    if isinstance(trafaret, t.Email):
        return fake.email()

    elif isinstance(trafaret, t.URL):
        return fake.url()

    elif isinstance(trafaret, t.String):
        return fake.name()

    elif isinstance(trafaret, t.Int):
        return random.randint(1, 10)

    elif isinstance(trafaret, t.Float):
        return round(random.uniform(1.0, 100.0), 2)

    elif isinstance(trafaret, t.List):
        res = []
        for i in range(random.randint(1, 5)):
            res.append(generate(trafaret.trafaret))
        return res

    elif isinstance(trafaret, t.Dict):
        res = {}
        for key in trafaret.keys:
            res[key.name] = generate(key.trafaret)
        return res

    elif isinstance(trafaret, t.Or):
        return generate(random.choice(trafaret.trafarets))

    elif isinstance(trafaret, t.Null):
        return

    raise NotImplementedError(repr(trafaret))



import os.path

import trafaret
from django.apps import apps
from django.utils.module_loading import import_module

from . import fake
from .backends.base import CompileError

__all__ = ('all_template_classes', 'template_classes', 'all_layout_classes',
           'layout_classes', 'find_templates', 'get_layout', 'get_template',
           'TemplateConfigurationError')

all_layout_classes = []
layout_classes = []

all_template_classes = []
template_classes = []


class EmailError(Exception):
    pass


class LayoutNotFound(EmailError):
    pass


class TemplateNotFound(EmailError):
    pass


class TemplateConfigurationError(EmailError):
    pass


def get_layout(name):
    for cls in layout_classes:
        if cls.name == name:
            return cls
    raise LayoutNotFound(name)


def get_template(name):
    for cls in template_classes:
        if cls.name == name:
            return cls
    raise TemplateNotFound(name)


def in_module(cls, module):
    if cls.__module__ == module.__name__:
        return True
    return cls.__module__.startswith('{}.'.format(module.__name__))


def find_templates():
    template_classes.clear()
    layout_classes.clear()

    for app in apps.get_app_configs():
        if os.path.isfile(os.path.join(app.path, 'mails.py')):
            import_module('{}.mails'.format(app.name))

    for cls in all_template_classes:
        if cls.abstract:
            continue
        found = False
        for app in apps.get_app_configs():
            if in_module(cls, app.module):
                found = True
                break
        if found:
            template_classes.append(cls)

    for cls in all_layout_classes:
        if cls.abstract:
            continue
        found = False
        for app in apps.get_app_configs():
            if in_module(cls, app.module):
                found = True
                break
        if found:
            layout_classes.append(cls)


def template_compile_check(cls):
    kwargs = fake.generate(cls.kwargs)
    variables = cls.fake_variables()
    variables = {x.name: render_python_key(x, variables) for x in cls.variables.keys}
    instance = cls(None, _force_variables=variables, **kwargs)

    try:
        instance.compile()
    except CompileError as e:
        model = instance.model
        model.has_error = True
        model.enabled = False
        model.save_base(raw=True)
        raise


def render_python_key(x, variables=None):
    if variables is None:
        variables = {}

    if isinstance(x.trafaret, trafaret.Dict):
        value = {k.name: render_python_key(k, variables.get(x.name)) for k in x.trafaret.keys}
    else:
        value = variables.get(x.name, fake.generate(x.trafaret))

    return value


def render_react_key(x, variables=None):
    if variables is None:
        variables = {}

    if isinstance(x.trafaret, trafaret.Dict):
        value = [render_react_key(k, variables.get(x.name)) for k in x.trafaret.keys]
    else:
        value = variables.get(x.name, fake.generate(x.trafaret))

    return {
        'name': x.name,
        'type': repr(x.trafaret),
        'value': value,
        'valueType': x.trafaret.__class__.__name__.lower()
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-22 01:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('body', models.TextField()),
                ('version', models.IntegerField()),
                ('archived_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-version',),
            },
        ),
        migrations.CreateModel(
            name='TemplateModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(unique=True)),
                ('layout', models.SlugField(null=True)),
                ('subject', models.TextField(null=True)),
                ('body', models.TextField(null=True)),
                ('version', models.IntegerField(default=1)),
                ('enabled', models.BooleanField(default=False)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Template',
            },
        ),
        migrations.AddField(
            model_name='historicaltemplate',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='happymailer.TemplateModel'),
        ),
    ]



from django.db import models
from django.utils import timezone

__all__ = ('TemplateModel', 'HistoricalTemplate',)


class TemplateModelQueryset(models.QuerySet):
    def active(self):
        return self.filter(enabled=True, has_errors=False)


class TemplateModel(models.Model):
    """
    stores editable template data
    """
    name = models.SlugField(unique=True)
    layout = models.SlugField(null=True)
    subject = models.TextField(null=True)
    body = models.TextField(null=True)
    version = models.IntegerField(default=1)
    enabled = models.BooleanField(default=False)
    has_errors = models.BooleanField(default=False)
    created_at = models.DateTimeField(default=timezone.now)
    updated_at = models.DateTimeField(auto_now=True)

    objects = TemplateModelQueryset.as_manager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = 'Template'
        ordering = ('name',)


class HistoricalTemplate(models.Model):
    """
    stores previous versions of TemplateModel
    """
    template = models.ForeignKey(TemplateModel, related_name='history')
    layout = models.SlugField(null=True)
    subject = models.TextField(null=True)
    body = models.TextField()
    version = models.IntegerField()
    archived_at = models.DateTimeField(default=timezone.now)

    def __str__(self):
        return '%s' % self.version

    class Meta:
        ordering = ('-version',)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-11 15:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('happymailer', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicaltemplate',
            name='layout',
            field=models.SlugField(null=True),
        ),
        migrations.AddField(
            model_name='historicaltemplate',
            name='subject',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='templatemodel',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='templatemodel',
            name='has_errors',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='historicaltemplate',
            name='archived_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='historicaltemplate',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='history', to='happymailer.TemplateModel'),
        ),
    ]



from django import forms

from .models import TemplateModel
from .utils import layout_classes, template_classes


class TemplateAdminForm(forms.ModelForm):
    subject = forms.CharField(widget=forms.Textarea(attrs={'class': 'vLargeTextField'}))
    layout = forms.ChoiceField(choices=[])

    def __init__(self, *args, **kwargs):
        super(TemplateAdminForm, self).__init__(*args, **kwargs)
        self.fields['layout'].choices = [(cls.name, cls.description or cls.name) for cls in layout_classes]

    class Meta:
        model = TemplateModel
        fields = ('layout', 'subject', 'body', 'enabled',)


class FakedataForm(forms.Form):
    layout = forms.CharField()
    template = forms.CharField()
    body = forms.CharField(required=False)
    subject = forms.CharField(required=False)
    variables = forms.CharField(required=False)
    send_test = forms.BooleanField(required=False, initial=False)

    def __init__(self, *args, **kwargs):
        super(FakedataForm, self).__init__(*args, **kwargs)
        self.fields['layout'].choices = [(cls.name, cls.name) for cls in layout_classes]
        self.fields['template'].choices = [(cls.name, cls.name) for cls in template_classes]


class ImportForm(forms.Form):
    import_file = forms.FileField()


from setuptools import setup, find_packages

setup(
    name='django-happymailer',
    version='0.1.7',
    description='django email templates manager',
    author='Victor Kotcheruba',
    author_email='barbuzaster@gmail.com',
    maintainer='Dmitrii Gerasimenko',
    maintainer_email='kiddima@gmail.com',
    include_package_data=True,
    packages=find_packages(exclude=['django_happymailer','dummy','dummy2']),
    install_requires=[
        'django >= 1.9',
        'django-import-export == 0.4.5',
        'fake-factory == 0.5.7',
        'html2text == 2016.5.29',
        'trafaret >= 0.7',
        'six',
    ]
)




import json

from django.conf import settings
from django.conf.urls import url
from django.contrib import admin, messages
from django.contrib.admin.utils import unquote
from django.core.serializers.json import DjangoJSONEncoder
from django.core.urlresolvers import reverse
from django.db import transaction
from django.http import Http404, JsonResponse, HttpResponse, HttpResponseRedirect
from django.template.exceptions import TemplateSyntaxError
from django.template.response import TemplateResponse

from . import fake
from .backends.base import CompileError
from .forms import TemplateAdminForm, FakedataForm, ImportForm
from .models import TemplateModel, HistoricalTemplate
from .utils import layout_classes, template_classes, get_template, get_layout, render_react_key, template_compile_check


@admin.register(TemplateModel)
class TemplateAdmin(admin.ModelAdmin):
    list_display = ('name', 'enabled', 'has_errors', 'version', 'subject', 'updated_at')
    readonly_fields = ('name',)
    form = TemplateAdminForm
    fields = ('name', 'enabled', 'has_errors', 'layout', 'subject', 'body',)

    import_template_name = 'admin/happymailer/templatemodel/import.html'

    def get_model_info(self):
        opts = self.model._meta
        return (opts.app_label, opts.model_name,)

    def get_urls(self):
        urls = super(TemplateAdmin, self).get_urls()
        admin_view = self.admin_site.admin_view
        info = self.get_model_info()
        extras = [
            url(r'^preview/$', admin_view(self.preview_action), name='%s_%s_preview' % info),
            url(r'^([^/]+)/version/([^/]+)/$', admin_view(self.version_action), name='%s_%s_version' % info),
            url(r'^export/$', admin_view(self.export_action), name='%s_%s_export' % info),
            url(r'^import/$', admin_view(self.import_action), name='%s_%s_import' % info),
        ]
        return extras + urls

    def export_action(self, request):
        response = HttpResponse(content_type='application/json')
        response['Content-Disposition'] = 'attachment; filename=templatemodel_export.json'

        items = [{
            'name': t.name,
            'layout': t.layout,
            'subject': t.subject,
            'body': t.body,
            'version': t.version,
            'enabled': t.enabled,
            'has_errors': t.has_errors,
            'created_at': t.created_at,
            'updated_at': t.updated_at,
            'history': [{
                'layout': h.layout,
                'subject': h.subject,
                'body': h.body,
                'version': h.version,
                'archived_at': h.archived_at,
            } for h in t.history.all()]
        } for t in TemplateModel.objects.all()]

        json.dump(items, response, cls=DjangoJSONEncoder, indent=2, sort_keys=True)
        return response

    def import_action(self, request):
        context = {}
        form = ImportForm(request.POST or None, request.FILES or None)

        if request.POST and form.is_valid():
            import_file = form.cleaned_data['import_file']
            data = json.loads(import_file.read().decode('utf-8'))

            created_cnt = 0
            updated_cnt = 0
            with transaction.atomic():
                for item in data:
                    item_defaults = dict(**item)
                    del item_defaults['history']

                    template, created = TemplateModel.objects.update_or_create(
                        name=item['name'],
                        defaults=item_defaults,
                    )

                    if created:
                        created_cnt += 1
                    else:
                        updated_cnt += 1

                    template.history.all().delete()
                    for h in item['history']:
                        template.history.create(**h)

                    try:
                        template_cls = get_template(template.name)
                        template_compile_check(template_cls)
                    except CompileError as e:
                        messages.warning(request, '%s got compile error: %r' % (template.name, str(e)))

            messages.info(request, '%d templates added, %d templates updated' % (created_cnt, updated_cnt))

            url = reverse('admin:%s_%s_changelist' % self.get_model_info(),
                          current_app=self.admin_site.name)
            return HttpResponseRedirect(url)

        context.update(self.admin_site.each_context(request))

        context['form'] = form
        context['opts'] = self.model._meta

        # request.current_app = self.admin_site.name
        return TemplateResponse(request, [self.import_template_name], context)

    def version_action(self, request, object_id, version):
        try:
            obj = HistoricalTemplate.objects.get(
                template=object_id,
                version=version,
            )
        except HistoricalTemplate.DoesNotExist:
            return JsonResponse({'version': 'Unknown version'}, status=403)

        return JsonResponse({
            'data': {
                'version': obj.version,
                'layout': obj.layout,
                'subject': obj.subject,
                'body': obj.body or '',
            }
        })

    def preview_action(self, request):
        form = FakedataForm(request.POST)
        if not form.is_valid():
            return JsonResponse({'form': form.errors}, status=400)

        variables = json.loads(form.cleaned_data['variables'])
        template_cls = get_template(form.cleaned_data['template'])
        layout_cls = get_layout(form.cleaned_data['layout'])
        kwargs = fake.generate(template_cls.kwargs)
        send_test = form.cleaned_data['send_test']

        recipient = '{} <{}>'.format(request.user.get_full_name(), request.user.email) if send_test else None

        tmpl = template_cls(recipient, _force_layout_cls=layout_cls, _force_variables=variables, **kwargs)
        tmpl.body = form.cleaned_data['body']
        tmpl.subject = "Test: {}".format(form.cleaned_data['subject'])

        try:
            compiled = tmpl.compile()
        except (CompileError, TemplateSyntaxError) as e:
            return JsonResponse({
                'template': str(e)
            }, status=400)

        if send_test:
            tmpl.send(force=True)

        return JsonResponse({
            'html': compiled,
            'email': recipient,
        })


    def get_actions(self, request):
        return None

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):

        if object_id:
            obj = self.get_object(request, admin.utils.unquote(object_id))

            ModelForm = self.get_form(request, obj)
            if request.method == 'POST':
                form = ModelForm(request.POST, request.FILES, instance=obj)
                if form.is_valid():
                    new_object = form.save()
                    return JsonResponse({'success': True})
                else:
                    return JsonResponse({'success': False, 'errors': form.errors})

            template = None
            for cls in template_classes:
                if cls.name == obj.name:
                    template = cls

            if not template:
                raise Http404()

            variables = template.fake_variables()

            history_options = [{
                'value': v.version,
                'label': '{version} (from {archived_at})'.format(
                    version=v.version, archived_at=v.archived_at.strftime('%Y-%m-%d %H:%M:%S'))}
                for v in obj.history.order_by('-version').all()]

            extra_context = dict(
                extra_context or {},
                happymailer_config=json.dumps({
                    'staticUrl': settings.STATIC_URL + 'happymailer/',
                    'template': {
                        'pk': obj.pk,
                        'template': obj.name,
                        'body': obj.body or '',
                        'layout': obj.layout or layout_classes[0].name,
                        'enabled': obj.enabled,
                        'subject': obj.subject or '',
                    },
                    'history': history_options,
                    'changelistUrl': reverse('admin:happymailer_templatemodel_changelist'),
                    'changeUrl': reverse('admin:happymailer_templatemodel_change', args=[obj.pk]),
                    'previewUrl': reverse('admin:happymailer_templatemodel_preview'),
                    'layouts': [{'value': cls.name, 'label': cls.description or cls.name}
                                for cls in layout_classes],
                    'variables': [render_react_key(x, variables) for x in template.variables.keys],
                }).replace('<', '\\u003C'),
            )

        return super(TemplateAdmin, self).changeform_view(request, object_id, form_url, extra_context)



import six
import trafaret as t
from django.test import SimpleTestCase

from ..fake import generate


class FakeCase(SimpleTestCase):
    def test_string(self):
        self.assertIsInstance(generate(t.String()), six.text_type)

    def test_email(self):
        self.assertIn('@', generate(t.Email()))

    def test_url(self):
        self.assertIn('://', generate(t.URL()))

    def test_int(self):
        self.assertIsInstance(generate(t.Int()), int)

    def test_list(self):
        lst = generate(t.List[t.String()])
        self.assertIsInstance(lst, list)
        for i in lst:
            self.assertIsInstance(i, six.text_type)

    def test_dict(self):
        dct = generate(t.Dict(foo=t.String(), bar=t.Int()))
        self.assertSetEqual(set(dct.keys()), {'foo', 'bar'})
        self.assertIsInstance(dct['foo'], six.text_type)
        self.assertIsInstance(dct['bar'], int)
